#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright 2011 VPAC <http://www.vpac.org>
# Copyright 2012-2024 Marcus Furlong <furlongm@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 3 only.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>

from geoip2 import database
from util import is_truthy


class GeoipDBLoader(object):

    def __init__(self, settings):
        geoip_data = settings.get('geoip_data')
        enable_maps = is_truthy(settings.get('enable_maps', False))
        self.gi = False
        if enable_maps and geoip_data:
            self.gi = database.Reader(geoip_data)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright 2011 VPAC <http://www.vpac.org>
# Copyright 2012-2024 Marcus Furlong <furlongm@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 3 only.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>

import logging
from ipaddress import ip_address
from vpns.openvpn.management_connection import ManagementConnection


class VPNDisconnector(object):

    def __init__(self, vpns, **kwargs):
        self.vpns = dict(vpns)
        self.check_disconnects(**kwargs)

    def check_disconnects(self, **kwargs):
        vpn_id = kwargs.get('vpn_id')
        if vpn_id:
            vpn = self.vpns[vpn_id]
            disconnection_allowed = vpn.get('show_disconnect')
            if disconnection_allowed:
                self.disconnect_client(vpn, **kwargs)

    def disconnect_client(self, vpn, **kwargs):
        connection = ManagementConnection(vpn)
        connection.connect()
        if connection.is_connected():
            name = vpn.get('name')
            version = vpn.get('version')
            command = False
            client_id = None
            if kwargs.get('client_id'):
                client_id = kwargs.get('client_id')
            if client_id and version.major == 2 and version.minor >= 4:
                logging.info(f'[{name}] Disconnecting client id `{client_id}`')
                command = f'client-kill {client_id}'
            else:
                ip = ip_address(kwargs.get('ip'))
                port = kwargs.get('port')
                if ip and port:
                    logging.info(f'[{name}] Disconnecting client `{ip}:{port}`')
                    command = f'kill {ip}:{port}'
            if command:
                connection.send_command(command)
            connection.disconnect()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright 2011 VPAC <http://www.vpac.org>
# Copyright 2012-2024 Marcus Furlong <furlongm@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 3 only.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>

import logging
import re
import semver
import string
from collections import deque
from ipaddress import ip_address
from geoip2.errors import AddressNotFoundError
from pprint import pformat
from util import get_date
from vpns.openvpn.management_connection import ManagementConnection


class VPNDataCollector(object):

    def __init__(self, vpns, gi, **kwargs):
        self.vpns = vpns
        for _, vpn in list(self.vpns.items()):
            self.collect_data(vpn, gi)

    def collect_data(self, vpn, gi):
        connection = ManagementConnection(vpn)
        connection.connect()
        if connection.is_connected():
            full_version = connection.send_command('version')
            release = self.parse_version(full_version)
            version = semver.parse_version_info(release.split(' ')[1])
            vpn['release'] = release
            vpn['version'] = version
            state = connection.send_command('state')
            vpn['state'] = self.parse_state(state)
            stats = connection.send_command('load-stats')
            vpn['stats'] = self.parse_stats(stats)
            status = connection.send_command('status 3')
            vpn['sessions'] = self.parse_status(status, version, gi)
        connection.disconnect()

    @staticmethod
    def parse_state(data):
        state = {}
        for line in data.splitlines():
            parts = line.split(',')
            logging.debug(f'=== begin split line\n{parts}\n=== end split line')
            if parts[0].startswith('>INFO') or \
               parts[0].startswith('END') or \
               parts[0].startswith('>CLIENT'):
                continue
            else:
                state['up_since'] = get_date(date_string=parts[0], uts=True)
                state['connected'] = parts[1]
                state['success'] = parts[2]
                if parts[3]:
                    state['local_ip'] = ip_address(parts[3])
                else:
                    state['local_ip'] = ''
                if parts[4]:
                    state['remote_ip'] = ip_address(parts[4])
                    state['mode'] = 'Client'
                else:
                    state['remote_ip'] = ''
                    state['mode'] = 'Server'
        return state

    @staticmethod
    def parse_stats(data):
        stats = {}
        line = re.sub('SUCCESS: ', '', data)
        parts = line.split(',')
        logging.debug(f'=== begin split line\n{parts}\n=== end split line')
        stats['nclients'] = int(re.sub('nclients=', '', parts[0]))
        stats['bytesin'] = int(re.sub('bytesin=', '', parts[1]))
        stats['bytesout'] = int(re.sub('bytesout=', '', parts[2]).replace('\r\n', ''))
        return stats

    def parse_status(self, data, version, gi):
        client_section = False
        routes_section = False
        sessions = {}
        client_session = {}

        for line in data.splitlines():
            parts = deque(line.split('\t'))
            logging.debug(f'=== begin split line\n{parts}\n=== end split line')

            if parts[0].startswith('END'):
                break
            if parts[0].startswith('TITLE') or \
               parts[0].startswith('GLOBAL') or \
               parts[0].startswith('TIME'):
                continue
            if parts[0] == 'HEADER':
                if parts[1] == 'CLIENT_LIST':
                    client_section = True
                    routes_section = False
                if parts[1] == 'ROUTING_TABLE':
                    client_section = False
                    routes_section = True
                continue

            if parts[0].startswith('TUN') or \
               parts[0].startswith('TCP') or \
               parts[0].startswith('Auth'):
                parts = parts[0].split(',')
            if parts[0] == 'TUN/TAP read bytes':
                client_session['tuntap_read'] = int(parts[1])
                continue
            if parts[0] == 'TUN/TAP write bytes':
                client_session['tuntap_write'] = int(parts[1])
                continue
            if parts[0] == 'TCP/UDP read bytes':
                client_session['tcpudp_read'] = int(parts[1])
                continue
            if parts[0] == 'TCP/UDP write bytes':
                client_session['tcpudp_write'] = int(parts[1])
                continue
            if parts[0] == 'Auth read bytes':
                client_session['auth_read'] = int(parts[1])
                sessions['Client'] = client_session
                continue

            if client_section:
                session = {}
                parts.popleft()
                common_name = parts.popleft()
                remote_str = parts.popleft()
                if remote_str.count(':') == 1:
                    remote, port = remote_str.split(':')
                elif '(' in remote_str:
                    remote, port = remote_str.split('(')
                    port = port[:-1]
                else:
                    remote = remote_str
                    port = None
                remote_ip = ip_address(remote)
                session['remote_ip'] = remote_ip
                if port:
                    session['port'] = int(port)
                else:
                    session['port'] = ''
                if session['remote_ip'].is_private:
                    session['location'] = 'RFC1918'
                elif session['remote_ip'].is_loopback:
                    session['location'] = 'loopback'
                else:
                    try:
                        if gi:
                            gir = gi.city(str(session.get('remote_ip')))
                            session['location'] = gir.country.iso_code
                            session['region'] = gir.subdivisions.most_specific.iso_code
                            session['city'] = gir.city.name
                            session['country'] = gir.country.name
                            session['longitude'] = gir.location.longitude
                            session['latitude'] = gir.location.latitude
                    except AddressNotFoundError as e:
                        logging.warning(e)
                    except SystemError:
                        pass
                local_ipv4 = parts.popleft()
                if local_ipv4:
                    session['local_ip'] = ip_address(local_ipv4)
                else:
                    session['local_ip'] = ''
                if version.major >= 2 and version.minor >= 4:
                    local_ipv6 = parts.popleft()
                    if local_ipv6:
                        session['local_ip'] = ip_address(local_ipv6)
                session['bytes_recv'] = int(parts.popleft())
                session['bytes_sent'] = int(parts.popleft())
                parts.popleft()
                session['connected_since'] = get_date(parts.popleft(), uts=True)
                username = parts.popleft()
                if username != 'UNDEF':
                    session['username'] = username
                else:
                    session['username'] = common_name
                if version.major == 2 and version.minor >= 4:
                    session['client_id'] = parts.popleft()
                    session['peer_id'] = parts.popleft()
                sessions[str(session['local_ip'])] = session

            if routes_section:
                local_ip = parts[1]
                remote_ip = parts[3]
                last_seen = get_date(parts[5], uts=True)
                if sessions.get(local_ip):
                    sessions[local_ip]['last_seen'] = last_seen
                elif self.is_mac_address(local_ip):
                    matching_local_ips = [sessions[s]['local_ip']
                                          for s in sessions if remote_ip ==
                                          self.get_remote_address(sessions[s]['remote_ip'], sessions[s]['port'])]
                    if len(matching_local_ips) == 1:
                        local_ip = f'{matching_local_ips[0]}'
                        if sessions[local_ip].get('last_seen'):
                            prev_last_seen = sessions[local_ip].get('last_seen')
                            if prev_last_seen < last_seen:
                                sessions[local_ip]['last_seen'] = last_seen
                        else:
                            sessions[local_ip]['last_seen'] = last_seen

        if sessions:
            pretty_sessions = pformat(sessions)
            logging.debug(f'=== begin sessions\n{pretty_sessions}\n=== end sessions')
        else:
            logging.debug('no sessions found')

        return sessions

    @staticmethod
    def parse_version(data):
        for line in data.splitlines():
            if line.startswith('OpenVPN'):
                return line.replace('OpenVPN Version: ', '')

    @staticmethod
    def is_mac_address(s):
        return len(s) == 17 and \
            len(s.split(':')) == 6 and \
            all(c in string.hexdigits for c in s.replace(':', ''))

    @staticmethod
    def get_remote_address(ip, port):
        if port:
            return f'{ip}:{port}'
        else:
            return f'{ip}'



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright 2011 VPAC <http://www.vpac.org>
# Copyright 2012-2024 Marcus Furlong <furlongm@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 3 only.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>

import logging
import os
import secrets
import sys
from datetime import datetime
from flask import Flask, request, render_template
from flask_wtf import CSRFProtect
from humanize import naturalsize
from pprint import pformat

cwd = os.path.dirname(__file__)
os.chdir(cwd)
sys.path.append(cwd)
from config.loader import ConfigLoader                    # noqa
from vpns.openvpn.data_collector import VPNDataCollector  # noqa
from vpns.openvpn.disconnector import VPNDisconnector     # noqa
from location_data.maxmind.geoip import GeoipDBLoader     # noqa
from util import is_truthy                                # noqa

logging.basicConfig(stream=sys.stderr, format='[%(asctime)s] [%(process)d] [%(levelname)s] %(message)s')
logging.getLogger().setLevel(logging.INFO)


def openvpn_monitor_wsgi():

    app = Flask(__name__)
    app.url_map.strict_slashes = False
    csrf = CSRFProtect(app)
    csrf.init_app(app)
    secret_key = secrets.token_hex(16)
    app.secret_key = secret_key

    if app.debug:
        logging.getLogger().setLevel(logging.DEBUG)

    config_file = os.getenv('OPENVPNMONITOR_CONFIG_FILE', '/etc/openvpn-monitor/openvpn-monitor.conf')
    config = ConfigLoader(config_file)
    settings = config.settings
    loaded_vpns = config.vpns
    geoip_db = GeoipDBLoader(settings)

    @app.template_filter()
    def get_formatted_time_now(datetime_format):
        return datetime.now().strftime(datetime_format)

    @app.template_filter()
    def get_vpn_anchor(vpn):
        if vpn.get('name'):
            return vpn.get('name').lower().replace(' ', '_')

    @app.template_filter()
    def get_naturalsize(nbytes):
        if nbytes:
            return naturalsize(nbytes, binary=True)

    @app.template_filter()
    def get_full_location(session):
        full_location = ''
        location = session.get('location')
        if location:
            if location in ['RFC1918', 'loopback']:
                city = location
                country = 'Internet'
                full_location = f'{city}, {country}'
            else:
                if session.get('country'):
                    country = session.get('country')
                    full_location = country
                if session.get('region'):
                    region = session.get('region')
                    full_location = f'{region}, {full_location}'
                if session.get('city'):
                    city = session.get('city')
                    full_location = f'{city}, {full_location}'
        return full_location

    @app.template_filter()
    def get_flag(session):
        flag = ''
        location = session.get('location')
        if location:
            if location in ['RFC1918', 'loopback']:
                flag = 'images/flags/rfc.png'
            else:
                flag = f'images/flags/{location.lower()}.png'
        return flag

    @app.template_filter()
    def get_vpn_error(vpn):
        name = vpn.get('name')
        host = vpn.get('host')
        port = vpn.get('port')
        socket = vpn.get('socket')
        error = vpn.get('error')
        if host and port:
            return f'{host}:{port} ({error})'
        elif socket:
            return f'{socket} ({error})'
        else:
            logging.error(f'unknown error with vpn {name} - {error}')
            return f'network or unix socket ({error})'

    @app.template_filter()
    def get_session_headers(vpn_mode):
        server_headers = [
            'Username / Hostname',
            'VPN IP',
            'Remote IP',
            'Location',
            'Bytes In',
            'Bytes Out',
            'Connected Since',
            'Last Ping',
            'Time Online'
        ]
        client_headers = [
            'Tun-Tap-Read',
            'Tun-Tap-Write',
            'TCP-UDP-Read',
            'TCP-UDP-Write',
            'Auth-Read'
        ]
        if vpn_mode == 'Client':
            headers = client_headers
        elif vpn_mode == 'Server':
            headers = server_headers
        return headers

    @app.template_filter()
    def get_total_connected_time(connected_since):
        return str(datetime.now() - connected_since)[:-7]

    @app.context_processor
    def inject_settings():
        site = settings.get('site', 'Example')
        logo = settings.get('logo')
        enable_maps = is_truthy(settings.get('enable_maps', False))
        maps_height = settings.get('maps_height', 500)
        latitude = settings.get('latitude', 40.72)
        longitude = settings.get('longitude', -74)
        datetime_format = settings.get('datetime_format', '%d/%m/%Y %H:%M:%S')
        return dict(
            site=site,
            logo=logo,
            enable_maps=enable_maps,
            maps_height=maps_height,
            latitude=latitude,
            longitude=longitude,
            datetime_format=datetime_format,
        )

    @app.route('/', methods=['GET', 'POST'])
    def handle_root():
        vpn_data = VPNDataCollector(loaded_vpns, geoip_db.gi)
        vpns = vpn_data.vpns.items()
        pretty_vpns = pformat((dict(vpns)))
        logging.debug(f'=== begin vpns\n{pretty_vpns}\n=== end vpns')
        if request.method == 'GET':
            return render_template('base.html', vpns=vpns)
        elif request.method == 'POST':
            vpn_id = request.form.get('vpn_id')
            ip = request.form.get('ip')
            port = request.form.get('port')
            client_id = request.form.get('client_id')
            VPNDisconnector(
                vpns=vpns,
                vpn_id=vpn_id,
                ip=ip,
                port=port,
                client_id=client_id,
            )
            return render_template('base.html', vpns=vpns)

    return app


application = openvpn_monitor_wsgi()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright 2011 VPAC <http://www.vpac.org>
# Copyright 2012-2024 Marcus Furlong <furlongm@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 3 only.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>

import logging
from datetime import datetime


def get_date(date_string, uts=False):
    if not uts:
        return datetime.strptime(date_string, '%a %b %d %H:%M:%S %Y')
    else:
        return datetime.fromtimestamp(float(date_string))


def is_truthy(s):
    return s in ['True', 'true', 'Yes', 'yes', True]


def multiline_info_log(s):
    for line in s.splitlines():
        logging.info(line)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright 2011 VPAC <http://www.vpac.org>
# Copyright 2012-2024 Marcus Furlong <furlongm@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 3 only.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>

import configparser
import logging
import os
from collections import OrderedDict
from pprint import pformat
from util import is_truthy, multiline_info_log


class ConfigLoader(object):

    def __init__(self, config_file):
        self.settings = {}
        self.vpns = OrderedDict()
        self.load_config_file(config_file)
        logging.debug(f'=== begin section\n{self.settings}\n=== end section')
        logging.debug(f'=== begin section\n{self.vpns}\n=== end section')

    def load_config_file(self, config_file):
        if os.path.isfile(config_file) and os.access(config_file, os.R_OK):
            logging.info(f'Using config file: {config_file}')
        else:
            logging.error(f'Config file does not exist or is unreadable: {config_file}')
            config_file = os.path.join(os.path.abspath(os.getcwd()), 'openvpn-monitor.conf')
            logging.warning(f'Trying {config_file} as config file')
        config = configparser.RawConfigParser()
        contents = config.read(config_file)
        if contents:
            self.parse_config(config)
        else:
            logging.error(f'No settings found in config file: {config_file}')
            logging.warning(f'Falling back to default settings')
            self.load_default_settings()

    def parse_config(self, config):
        for section in config.sections():
            if section.lower() == 'openvpn-monitor':
                self.parse_global_section(config)
            else:
                self.parse_vpn_section(config, section)
        multiline_info_log(f'Using loaded config:\n{pformat(config._sections)}')

    def load_default_settings(self):
        self.settings = {'site': 'Default Site',
                         'enable_maps': False,
                         'geoip_data': '/usr/share/GeoIP/GeoLite2-City.mmdb',
                         'datetime_format': '%d/%m/%Y %H:%M:%S'}
        self.vpns['Default VPN'] = {'name': 'default',
                                    'host': 'localhost',
                                    'port': '5555',
                                    'password': '',
                                    'show_disconnect': False}
        logging.info('Using default settings:')
        multiline_info_log(pformat(self.settings))
        multiline_info_log(pformat(self.vpns))

    def parse_global_section(self, config):
        global_vars = [
            'site',
            'logo',
            'latitude',
            'longitude',
            'enable_maps',
            'maps_height',
            'geoip_data',
            'datetime_format'
        ]
        for var in global_vars:
            try:
                self.settings[var] = config.get('openvpn-monitor', var)
            except configparser.NoOptionError:
                pass

    def parse_vpn_section(self, config, section):
        self.vpns[section] = {}
        vpn = self.vpns[section]
        options = config.options(section)
        for option in options:
            try:
                vpn[option] = config.get(section, option)
                if vpn[option] == -1:
                    logging.warning(f'config: skipping {option}')
            except configparser.Error as e:
                logging.warning(f'config: {e} on option {option}: ')
                vpn[option] = None
        vpn['show_disconnect'] = is_truthy(vpn.get('show_disconnect', False))



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright 2011 VPAC <http://www.vpac.org>
# Copyright 2012-2024 Marcus Furlong <furlongm@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 3 only.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>

import logging
import re
import socket
import ssl


class ManagementConnection(object):

    def __init__(self, vpn_config):
        self.__vpn_config = vpn_config
        self.__name = self.__vpn_config.get('name')
        self.__timeout = 3
        self.__error = False
        self.__socket = False
        self.__vpn_config['management_connection_successful'] = False

    def is_connected(self):
        return self.__socket

    def connect(self):
        try:
            self.__connect()
            self.__authenticate()
            self.__vpn_config['management_connection_successful'] = True
        except socket.timeout as e:
            self.__handle_connect_error(e, 'socket timeout')
        except socket.error as e:
            self.__handle_connect_error(e, 'socket error')
        except ssl.SSLError as e:
            self.__handle_connect_error(e, 'ssl error')
        except Exception as e:
            self.__handle_connect_error(e, 'unexpected error')

    def __handle_connect_error(self, error, msg):
        self.__error = error
        self.__vpn_config['error'] = self.__error
        self.__close()
        logging.warning(f'{msg}: {error}')

    def send_command(self, command):
        logging.info(f'[{self.__name}] Sending openvpn management command: `{command.rstrip()}`')
        self.__send(f'{command}\n')
        if command.startswith('kill') or command.startswith('client-kill'):
            return
        return self.__wait_for_data(command=command)

    def __wait_for_data(self, password=None, command=None):
        data = ''
        while 1:
            try:
                socket_data = self.__recv(1024)
            except TimeoutError:
                logging.error(f'[{self.__name}] Timeout receiving data')
                break
            socket_data = re.sub('>INFO(.)*\r\n', '', socket_data)
            data += socket_data
            if data.endswith('ENTER PASSWORD:'):
                if password:
                    self.__send(f'{password}\n')
                else:
                    logging.warning(f'[{self.__name}] Password requested but no password supplied by configuration')
            if data.endswith('SUCCESS: password is correct\r\n'):
                break
            if command == 'load-stats' and data != '':
                break
            if command == 'quit':
                break
            elif data.endswith("\nEND\r\n"):
                break
        logging.debug(f'[{self.__name}] === begin raw data\n{data}\n=== end raw data')
        return data

    def __send(self, data):
        self.__socket.send(bytes(data, 'utf-8'))

    def __recv(self, length):
        return self.__socket.recv(length).decode('utf-8')

    def disconnect(self):
        if self.__socket:
            self.send_command('quit')
            self.__close()

    def __close(self):
        if self.__socket:
            self.__socket.shutdown(socket.SHUT_RDWR)
            self.__socket.close()
            self.__socket = False

    def __authenticate(self):
        if (self.__vpn_config.get('password')):
            self.__wait_for_data(password=self.__vpn_config.get('password'))

    def __connect(self):
        if self.__is_tls_socket():
            self.__connect_tls()
        elif self.__is_tcp_socket():
            self.__connect_tcp()
        elif self.__is_unix_socket():
            self.__connect_unix()
        else:
            raise Exception('Unknown socket type')

    def __is_tls_socket(self):
        return self.__vpn_config.get('host') and self.__vpn_config.get('ssl')

    def __connect_tls(self):
        logging.info(f'[{self.__name}] Initiating TLS socket connection')
        context = self.__create_tls_context()
        self.__connect_tcp()
        self.__socket = context.wrap_socket(self.__socket)

    def __create_tls_context(self):
        context = ssl.SSLContext(ssl.PROTOCOL_TLS_CLIENT)
        context.options |= ssl.OP_NO_TLSv1
        context.options |= ssl.OP_NO_TLSv1_1
        if (self.__vpn_config.get('ssl') == 'any-cert'):
            context.check_hostname = False
            context.verify_mode = ssl.CERT_NONE
        return context

    def __is_tcp_socket(self):
        return self.__vpn_config.get('host') and not self.__vpn_config.get('ssl')

    def __connect_tcp(self):
        host = self.__vpn_config['host']
        port = int(self.__vpn_config['port'])
        logging.info(f'[{self.__name}] Initiating TCP socket connection to {host}:{port}')
        self.__socket = socket.create_connection((host, port), self.__timeout)

    def __is_unix_socket(self):
        return bool(self.__vpn_config.get('socket'))

    def __connect_unix(self):
        self.__socket = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
        unix_socket = self.__vpn_config['socket']
        logging.info(f'[{self.__name}] Initiating UNIX socket connection to {unix_socket}')
        self.__socket.connect(unix_socket)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Copyright 2011 VPAC <http://www.vpac.org>
# Copyright 2012-2024 Marcus Furlong <furlongm@gmail.com>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 3 only.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>

import os
import sys
from setuptools import setup, find_packages

with open('VERSION.txt', 'r', encoding='utf_8') as v:
    version = v.readline().strip()

with open('README.md', 'r', encoding='utf_8') as r:
    long_description = r.read()

with open('requirements.txt', 'r', encoding='utf_8') as rt:
    install_requires = rt.read().splitlines()

package_files = []
for directory in ['openvpn_monitor/static', 'openvpn_monitor/templates']:
    for (path, directories, filenames) in os.walk(directory):
        for filename in filenames:
            trimmed_path = path.replace('openvpn_monitor/', '')
            package_files.append(os.path.join(trimmed_path, filename))

if sys.prefix == '/usr':
    conf_path = '/etc'
else:
    conf_path = sys.prefix

data_files = []
for dirpath, dirnames, filenames in os.walk('etc'):
    for i, dirname in enumerate(dirnames):
        if dirname.startswith('.'):
            del dirnames[i]
    if filenames:
        data_files.append(
            ['/etc/openvpn-monitor', [os.path.join(dirpath, f) for f in filenames]]
        )

setup(
    name='openvpn-monitor',
    version=version,
    author='Marcus Furlong',
    author_email='furlongm@gmail.com',
    description=('A simple web based openvpn monitor'),
    license='GPLv3',
    keywords='web openvpn monitor',
    url='http://openvpn-monitor.openbytes.ie',
    packages=find_packages(),
    package_data={'openvpn_monitor': package_files},
    install_requires=install_requires,
    long_description=long_description,
    long_description_content_type='text/markdown',
    data_files=data_files,
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Topic :: Internet :: WWW/HTTP :: WSGI :: Application',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
    ],
)
